@@ -24,28 +24,30 @@
 #include <gst/gst.h>
 #include <gconf/gconf-client.h>
 
+#include "gstswitchsrc.h"
+
 G_BEGIN_DECLS
 
 #define GST_TYPE_GCONF_AUDIO_SRC            (gst_gconf_audio_src_get_type ())
 #define GST_GCONF_AUDIO_SRC(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_GCONF_AUDIO_SRC, GstGConfAudioSrc))
-#define GST_GCONF_AUDIO_SRC_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_GCONF_AUDIO_SRC, GstGConfAudioSrc))
+#define GST_GCONF_AUDIO_SRC_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_GCONF_AUDIO_SRC, GstGConfAudioSrcClass))
 #define GST_IS_GCONF_AUDIO_SRC(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_GCONF_AUDIO_SRC))
 #define GST_IS_GCONF_AUDIO_SRC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_GCONF_AUDIO_SRC))
 
 typedef struct _GstGConfAudioSrc {
-  GstBin parent;
+  GstSwitchSrc parent;
 
   /* explicit pointers to stuff used */
   GConfClient *client;
-  GstElement *kid;
-  GstPad *pad;
+
+  guint gconf_notify_id;
 
   /* Current gconf string */
   gchar *gconf_str;
 } GstGConfAudioSrc;
 
 typedef struct _GstGConfAudioSrcClass {
-  GstBinClass parent_class;
+  GstSwitchSrcClass parent_class;
 } GstGConfAudioSrcClass;
 
 GType   gst_gconf_audio_src_get_type   (void);
